@@ -34,9 +34,15 @@
         (include-list prj t))
       (list)))
 
-(tm-define (project-menu)
+(tm-define (project-list-menu)
   (buffer-list-menu (project-file-list)))
 
+(menu-bind project-menu
+  (if (== (project-get) (current-buffer))
+      (link preamble-menu)
+      ---)
+  (link project-list-menu))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Document style
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -117,7 +123,7 @@
 
 (menu-bind document-font-menu
   (-> "Text font"
-      ("Default" (init-default "font"))
+      ("Default" (init-default "font" "math-font" "prg-font"))
       ---
       ("Concrete" (init-env "font" "concrete"))
       (if (url-exists-in-tex? "pnr10.mf")
@@ -134,6 +140,8 @@
       ("Palatino" (init-env "font" "palatino"))
       ("Times" (init-env "font" "times"))
       ---
+      (if (font-exists-in-tt? "texgyrebonum-regular")
+          ("Bonum" (init-env "font" "bonum")))
       (if (font-exists-in-tt? "DejaVuSerif")
           ("Dejavu" (init-env "font" "dejavu")))
       ("Lucida" (init-env "font" "x-lucida"))
@@ -141,6 +149,8 @@
           ("Luxi" (init-env "font" "luxi")))
       (if (font-exists-in-tt? "texgyrepagella-regular")
           ("Pagella" (init-env "font" "pagella")))
+      (if (font-exists-in-tt? "texgyreschola-regular")
+          ("Schola" (init-env "font" "schola")))
       (if (font-exists-in-tt? "texgyretermes-regular")
           ("Termes" (init-env "font" "termes")))
       ("Utopia" (init-env "font" "x-utopia"))
@@ -220,6 +230,8 @@
           ("Apple symbols" (init-env "math-font" "math-apple")))
       (if (font-exists-in-tt? "Asana-Math")
           ("Asana" (init-env "math-font" "math-asana")))
+      (if (font-exists-in-tt? "texgyrebonum-math")
+          ("Bonum" (init-env "math-font" "math-bonum")))
       ("Concrete" (init-env "math-font" "concrete"))
       (if (font-exists-in-tt? "DejaVuSerif")
           ("Dejavu" (init-env "math-font" "math-dejavu")))
@@ -229,6 +241,8 @@
       (if (font-exists-in-tt? "texgyrepagella-math")
           ("Pagella" (init-env "math-font" "math-pagella")))
       ("Roman" (init-env "math-font" "roman"))
+      (if (font-exists-in-tt? "texgyreschola-math")
+          ("Schola" (init-env "math-font" "math-schola")))
       (if (font-exists-in-tt? "STIX-Regular")
           ("Stix" (init-env "math-font" "math-stix")))
       (if (font-exists-in-tt? "texgyretermes-math")
@@ -343,24 +357,63 @@
   ("Concrete" (init-font "concrete" "concrete"))
   ("Roman" (init-font "roman" "roman"))
   ("Stix" (init-font "stix" "math-stix"))
-  ---
-  ("Avant Garde" (init-env "font" "avant-garde"))
-  ("Bookman" (init-env "font" "bookman"))
-  ("Courier" (init-env "font" "courier"))
-  ("Helvetica" (init-env "font" "helvetica"))
-  ("NC Schoolbook" (init-env "font" "new-century-schoolbook"))
-  ("Palatino" (init-env "font" "palatino"))
-  ("Times" (init-env "font" "times"))
-  (if (or (font-exists-in-tt? "DejaVuSerif")
+  ;;---
+  ;;("Avant Garde" (init-env "font" "avant-garde"))
+  ;;("Bookman" (init-env "font" "bookman"))
+  ;;("Courier" (init-env "font" "courier"))
+  ;;("Helvetica" (init-env "font" "helvetica"))
+  ;;("NC Schoolbook" (init-env "font" "new-century-schoolbook"))
+  ;;("Palatino" (init-env "font" "palatino"))
+  ;;("Times" (init-env "font" "times"))
+  (if (or (font-exists-in-tt? "texgyrebonum-math")
           (font-exists-in-tt? "texgyrepagella-math")
+          (font-exists-in-tt? "texgyreschola-math")
           (font-exists-in-tt? "texgyretermes-math"))
       ---
-      (if (font-exists-in-tt? "DejaVuSerif")
-          ("Dejavu" (init-font "dejavu" "math-dejavu")))
+      (group "TeX Gyre")
+      (if (font-exists-in-tt? "texgyrebonum-math")
+          ("Bonum" (init-font "bonum" "math-bonum")))
       (if (font-exists-in-tt? "texgyrepagella-math")
           ("Pagella" (init-font "pagella" "math-pagella")))
+      (if (font-exists-in-tt? "texgyreschola-math")
+          ("Schola" (init-font "schola" "math-schola")))
       (if (font-exists-in-tt? "texgyretermes-math")
           ("Termes" (init-font "termes" "math-termes"))))
+  (if (or (font-exists-in-tt? "DejaVuSerif")
+          (font-exists-in-tt? "LinLibertine_R")
+          (font-exists-in-tt? "Optima")
+          (font-exists-in-tt? "Papyrus"))
+      ---
+      (if (font-exists-in-tt? "AmericanTypewriter")
+          ("American Typewriter" (init-env "font" "American Typewriter")))
+      (if (font-exists-in-tt? "Baskerville")
+          ("Baskerville" (init-env "font" "Baskerville")))
+      (if (font-exists-in-tt? "Chalkboard")
+          ("Chalkboard" (init-env "font" "Chalkboard")))
+      (if (font-exists-in-tt? "Chalkduster")
+          ("Chalkduster" (init-env "font" "Chalkduster")))
+      (if (font-exists-in-tt? "Cochin")
+          ("Cochin" (init-env "font" "Cochin")))
+      (if (font-exists-in-tt? "Cuprum-Regular")
+          ("Cuprum" (init-env "font" "Cuprum")))
+      (if (font-exists-in-tt? "DejaVuSerif")
+          ("Dejavu" (init-font "dejavu" "math-dejavu")))
+      (if (font-exists-in-tt? "Didot")
+          ("Didot" (init-env "font" "Didot")))
+      (if (font-exists-in-tt? "Essays1743")
+          ("Essays1743" (init-env "font" "Essays1743")))
+      (if (font-exists-in-tt? "MarkerFelt")
+          ("Marker Felt" (init-env "font" "Marker Felt")))
+      (if (font-exists-in-tt? "meyne_textur")
+          ("Meyne Textur" (init-env "font" "Meyne Textur")))
+      (if (font-exists-in-tt? "LinBiolinum_R")
+          ("Linux Biolinum" (init-env "font" "Linux Biolinum")))
+      (if (font-exists-in-tt? "LinLibertine_R")
+          ("Linux Libertine" (init-env "font" "Linux Libertine")))
+      (if (font-exists-in-tt? "Optima")
+          ("Optima" (init-env "font" "Optima")))
+      (if (font-exists-in-tt? "Papyrus")
+          ("Papyrus" (init-env "font" "Papyrus"))))
   (if (and (supports-chinese?) (== (get-init "language") "chinese"))
       ---
       (link document-short-chinese-font-menu))
@@ -375,7 +428,7 @@
       (link document-short-chinese-font-menu))
   ---
   (if (and (new-fonts?) (use-popups?))
-      ("Other" (open-font-selector)))
+      ("Other" (open-document-font-selector)))
   (if (not (and (new-fonts?) (use-popups?)))
       (-> "Other" (link document-font-menu))))
 
@@ -391,7 +444,7 @@
       ("Left aligned" (init-env "par-mode" "left"))
       ("Centered" (init-env "par-mode" "center"))
       ("Right aligned" (init-env "par-mode" "right")))
-  (-> "Hyphenation"
+  (-> "Line breaking"
       ("Default" (init-default "par-hyphen"))
       ---
       ("Normal" (init-env "par-hyphen" "normal"))
@@ -544,14 +597,19 @@
 ;; The Document -> Page menu
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
+(menu-bind page-rendering-menu
+  ("Paper" (init-page-rendering "paper"))
+  ("Papyrus" (init-page-rendering "papyrus"))
+  ("Automatic" (init-page-rendering "automatic"))
+  ("Beamer" (init-page-rendering "beamer"))
+  ("Book" (init-page-rendering "book"))
+  ("Panorama" (init-page-rendering "panorama")))
+
 (menu-bind document-page-menu
   (-> "Type"
-      ("Default" (init-default-page-medium))
+      ("Default" (init-default-page-rendering))
       ---
-      ("Paper" (init-page-medium "paper"))
-      ("Papyrus" (init-page-medium "papyrus"))
-      ("Automatic" (init-page-medium "automatic"))
-      ("Beamer" (init-page-medium "beamer")))
+      (link page-rendering-menu))
   (-> "Size"
       (link document-page-size-menu))
   (-> "Orientation"
@@ -647,6 +705,17 @@
 ;; The Document -> Color menu
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
+(define (set-background col)
+  (init-env-tree "bg-color" col))
+
+(define (set-background-pattern name)
+  (when (pair? name) (set! name (car name)))
+  (init-env-tree "bg-color" (tm-pattern name "" "")))
+
+(define (set-background-picture name)
+  (when (pair? name) (set! name (car name)))
+  (init-env-tree "bg-color" (tm-pattern name "100%" "100@")))
+
 (menu-bind document-foreground-color-menu
   ("Default" (init-default "color"))
   ---
@@ -661,8 +730,8 @@
   ---
   (pick-background "" (init-env-tree "bg-color" answer))
   ---
-  ("Palette" (interactive-background
-              (lambda (col) (init-env "bg-color" col)) '()))
+  ("Palette" (interactive-background set-background '()))
+  ("Pattern" (open-pattern-selector set-background "1cm"))
   ("Other" (init-interactive-env "bg-color")))
 
 (menu-bind document-colors-menu
@@ -736,7 +805,8 @@
   ;;(link remove-package-menu)
   ;;---
   ;;("Other" (interactive remove-style-package)))
-  (if (!= (get-init-tree "sectional-short-style") (tree 'macro "false"))
+  (if (and (not (project-attached?))
+           (!= (get-init-tree "sectional-short-style") (tree 'macro "false")))
       (-> "Part" (link document-part-menu)))
   (-> "Source"
       ("Edit source tree" (toggle-preamble))
@@ -758,12 +828,15 @@
       ("Default" (init-default "prog-scripts"))
       ---
       (link supported-scripts-menu))
-  (-> "Informative flags" (link document-informative-flags-menu)))
+  (-> "Informative flags" (link document-informative-flags-menu))
+  (if (== (get-preference "experimental encryption") "on")
+      (-> "Encryption" (link document-encryption-menu))))
 
 (menu-bind compressed-document-menu
   (-> "Style" (link document-style-menu))
   (link document-style-extra-menu)
-  (if (!= (get-init-tree "sectional-short-style") (tree 'macro "false"))
+  (if (and (not (project-attached?))
+           (!= (get-init-tree "sectional-short-style") (tree 'macro "false")))
       (-> "Part" (link document-part-menu)))
   (-> "Source"
       ("Edit source tree" (toggle-preamble))
@@ -787,7 +860,9 @@
       ("Default" (init-default "prog-scripts"))
       ---
       (link supported-scripts-menu))
-  (-> "Informative flags" (link document-informative-flags-menu)))
+  (-> "Informative flags" (link document-informative-flags-menu))
+  (if (== (get-preference "experimental encryption") "on")
+      (-> "Encryption" (link document-encryption-menu))))
 
 (menu-bind document-menu
   (if (use-menus?)
@@ -834,15 +909,21 @@
       (link document-page-size-menu))
   (-> (eval (string-append (get-init "font-base-size") " pt"))
       (link document-font-base-size-menu))
+  (-> (eval (upcase-first (get-init-page-rendering)))
+      (link page-rendering-menu))
   (-> (eval (upcase-first (get-init "page-orientation")))
       ("Portrait" (init-page-orientation "portrait"))
       ("Landscape" (init-page-orientation "landscape")))
   (-> (eval (number-columns-text (get-init "par-columns")))
       ("One column" (init-env "par-columns" "1"))
       ("Two columns" (init-env "par-columns" "2")))
+  (if (and (== (get-preference "experimental encryption") "on")
+	   (!= (get-init "encryption") ""))
+      (=> (balloon (icon "tm_lock_open.xpm") "Encryption")
+	  (link document-encryption-menu)))
   (-> (eval (upcase-first (get-init "language")))
       (link document-language-menu))
-  (-> (eval (upcase-first (get-init "font")))
+  (-> (eval (upcase-first (font-family-main (get-init "font"))))
       (link document-short-font-menu)))
 
 (tm-menu (standard-focus-menu t)
@@ -897,7 +978,7 @@
     (=> (balloon (eval (string-append (get-init "font-base-size") "pt"))
                  "Font size")
         (link document-font-base-size-menu))
-    (=> (balloon (eval (upcase-first (get-init "font")))
+    (=> (balloon (eval (upcase-first (font-family-main (get-init "font"))))
                  "Main document font")
         (link document-short-font-menu))
     (=> (balloon (icon (eval (current-page-icon))) "Page layout")
@@ -907,10 +988,11 @@
         ("One column" (init-env "par-columns" "1"))
         ("Two columns" (init-env "par-columns" "2"))
         ---
-        ("Paper" (init-page-medium "paper"))
-        ("Papyrus" (init-page-medium "papyrus"))
-        ("Automatic" (init-page-medium "automatic"))
-        ("Beamer" (init-page-medium "beamer")))
+	(link page-rendering-menu))
+    (if (and (== (get-preference "experimental encryption") "on")
+	     (!= (get-init "encryption") ""))
+	(=> (balloon (icon "tm_lock_open.xpm") "Encryption")
+	 (link document-encryption-menu)))
     (=> (balloon (icon (eval (current-language-icon))) "Document language")
         (link document-language-menu))))
 
